# -*- coding: utf-8 -*-
import re
EMAIL_REGEX =  re.compile(r"[\w\.-]+@[\w\.-]+")
NUMBERS_REGEX = re.compile(r"\d+")
PHONE_REGEX = re.compile(r"[\+\(]?[1-9][0-9 .\-\(\)]{8,}[0-9]")
SPECIAL_CHARACTERS_REGEX = re.compile(r"[^A-Za-z0-9 ]+")
EMOJI_REGEX = re.compile("["
                       u"\U0001F600-\U0001F64F"  # for emoticons
                       u"\U0001F300-\U0001F5FF"  # for symbols & pictographs
                       u"\U0001F680-\U0001F6FF"  # for transport & map symbols
                       u"\U0001F1E0-\U0001F1FF"  # for flags (iOS)
                       u"\U00002702-\U000027B0"
                       u"\U000024C2-\U0001F251"
                       "]+", flags=re.UNICODE)

DATE_REGEX = re.compile(r"([0-9]{2}\/[0-9]{2}\/[0-9]{4})|([0-9]{4}\/[0-9]{2}\/[0-9]{2})")
PUNCT_REGEX = re.compile(r"""[!"&'()*,-./:;?@[\\]^_`{|}]""")
# modified for full stops, question marks, commas, colons and semi-colons, exclamation marks and quotation marks.
MOST_COMMON_PUNCT_REGEX = re.compile(r"""[!"&',-.;?_`]""")
HTML_TAGS_REGEX = re.compile(r'<[^<]+?>')
# modified source :https://gist.github.com/dperini/729294
URL_PATTERN = re.compile(
    r"(?:^|(?<![\w\/\.]))"
    # protocol identifier
    # r"(?:(?:https?|ftp)://)"  <-- alt?
    r"(?:(?:https?:\/\/|ftp:\/\/|www\d{0,3}\.))"
    # user:pass authentication
    r"(?:\S+(?::\S*)?@)?" r"(?:"
    # IP address exclusion
    # private & local networks
    r"(?!(?:10|127)(?:\.\d{1,3}){3})"
    r"(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})"
    r"(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})"
    # IP address dotted notation octets
    # excludes loopback network 0.0.0.0
    # excludes reserved space >= 224.0.0.0
    # excludes network & broadcast addresses
    # (first & last IP address of each class)
    r"(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])"
    r"(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}"
    r"(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))"
    r"|"
    # host name
    r"(?:(?:[a-z\\u00a1-\\uffff0-9]-?)*[a-z\\u00a1-\\uffff0-9]+)"
    # domain name
    r"(?:\.(?:[a-z\\u00a1-\\uffff0-9]-?)*[a-z\\u00a1-\\uffff0-9]+)*"
    # TLD identifier
    r"(?:\.(?:[a-z\\u00a1-\\uffff]{2,}))" r")"
    # port number
    r"(?::\d{2,5})?"
    # resource path
    r"(?:\/[^\)\]\}\s]*)?",
    flags=re.UNICODE | re.IGNORECASE,
)

CURRENCY_REGEX = re.compile(
    r"[$¢£¤¥ƒ֏؋৲৳૱௹฿៛ℳ元円圆圓﷼\u20A0-\u20C0]\d+",
    flags=re.UNICODE)

CURRENCY_SYMB_REGEX = re.compile(
    r"[$¢£¤¥ƒ֏؋৲৳૱௹฿៛ℳ元円圆圓﷼\u20A0-\u20C0]",
    flags=re.UNICODE)

HASTAG_REGEX = re.compile(r"#\S+")
USER_HANDLES_REGEX = re.compile(r"@\S+")
# PHONE_REGEX = re.compile(
#     r"(?:^|(?<=[^\w)]))(\+?1[ .-]?)?(\(?\d{3}\)?[ .-]?)?(\d{3}[ .-]?\d{4})"
#     r"(\s?(?:ext\.?|[#x-])\s?\d{2,6})?(?:$|(?=\W))",
#     flags=re.UNICODE | re.IGNORECASE)

STREET_ADDRESS_REGEX  = re.compile(r"\d{1,4} [\w\s]{1,20}(?:street|st|avenue|ave|road|rd|highway|hwy|square|sq|trail|trl|drive|dr|court|ct|park|parkway|pkwy|circle|cir|boulevard|blvd)\W?`")
PoBOX_REGEX  = re.compile(r"(?i)P\.? ?O\.? Box \d+")
ZIP_REGEX  = re.compile(r"\b\d{5}(?:[-\s]\d{4})?\b")

BTC_ADDRESS_REGEX = re.compile(r"[13][a-km-zA-HJ-NP-Z1-9]{25,34}")
CreditCard_REGEX     = re.compile(r"(?:(?:(?:\d{4}[- ]?){3}\d{4}|\d{15,16}))")
VISACard_REGEX = re.compile(r"4\d{3}[\s-]?\d{4}[\s-]?\d{4}[\s-]?\d{4}")
MASTERCard_REGEX  = re.compile(r"5[1-5]\d{2}[\s-]?\d{4}[\s-]?\d{4}[\s-]?\d{4}")

# MD5,SHA REGEX
MD5_SHA_REGEX = re.compile(r"([0-9a-fA-F]{32})|([0-9a-fA-F]{40})|([0-9a-fA-F]{64})")




STOPWORDS = ['i', 'me', 'my', 'myself', 'we', 'our', 'ours', 'ourselves', 'you', "you're", "you've", "you'll", "you'd", 'your', 'yours', 'yourself', 'yourselves', 'he', 'him', 'his', 'himself', 'she', "she's", 'her', 'hers', 'herself', 'it', "it's", 'its', 'itself', 'they', 'them', 'their', 'theirs', 'themselves', 'what', 'which', 'who', 'whom', 'this', 'that', "that'll", 'these', 'those', 'am', 'is', 'are', 'was', 'were', 'be', 'been', 'being', 'have', 'has', 'had', 'having', 'do', 'does', 'did', 'doing', 'a', 'an', 'the', 'and', 'but', 'if', 'or', 'because', 'as', 'until', 'while', 'of', 'at', 'by', 'for', 'with', 'about', 'against', 'between', 'into', 'through', 'during', 'before', 'after', 'above', 'below', 'to', 'from', 'up', 'down', 'in', 'out', 'on', 'off', 'over', 'under', 'again', 'further', 'then', 'once', 'here', 'there', 'when', 'where', 'why', 'how', 'all', 'any', 'both', 'each', 'few', 'more', 'most', 'other', 'some', 'such', 'no', 'nor', 'not', 'only', 'own', 'same', 'so', 'than', 'too', 'very', 's', 't', 'can', 'will', 'just', 'don', "don't", 'should', "should've", 'now', 'd', 'll', 'm', 'o', 're', 've', 'y', 'ain', 'aren', "aren't", 'couldn', "couldn't", 'didn', "didn't", 'doesn', "doesn't", 'hadn', "hadn't", 'hasn', "hasn't", 'haven', "haven't", 'isn', "isn't", 'ma', 'mightn', "mightn't", 'mustn', "mustn't", 'needn', "needn't", 'shan', "shan't", 'shouldn', "shouldn't", 'wasn', "wasn't", 'weren', "weren't", 'won', "won't", 'wouldn', "wouldn't"]

AUTOMATED_READ_INDEX = {"1":"5-6 years (Kindergarten)","2":"6-7 years (First/Second Grade)","3":"7-9 years (Third Grade)","4":"9-10 years (Fourth Grade)","5":"10-11 years (Fifth Grade)","6":"11-12 years (Sixth Grade)","7":"12-13 years (Seventh Grade)","8":"13-14 years (Eighth Grade)","9":"14-15 years (Ninth Grade)","10":"15-16 years (Tenth Grade)","11":"16-17 years (Eleventh Grade)","12":"17-18 years (Twelfth grade)","13":"18-24 years (College student)","14":"24+ years (Professor)"}

CONTRACTIONS_DICT = {
  "ain't": "am not",
  "aren't": "are not",
  "can't": "cannot",
  "can't've": "cannot have",
  "'cause": "because",
  "could've": "could have",
  "couldn't": "could not",
  "couldn't've": "could not have",
  "didn't": "did not",
  "doesn't": "does not",
  "don't": "do not",
  "hadn't": "had not",
  "hadn't've": "had not have",
  "hasn't": "has not",
  "haven't": "have not",
  "he'd": "he would",
  "he'd've": "he would have",
  "he'll": "he will",
  "he'll've": "he will have",
  "he's": "he is",
  "how'd": "how did",
  "how'd'y": "how do you",
  "how'll": "how will",
  "how's": "how is",
  "I'd": "I would",
  "I'd've": "I would have",
  "I'll": "I will",
  "I'll've": "I will have",
  "I'm": "I am",
  "I've": "I have",
  "isn't": "is not",
  "it'd": "it had",
  "it'd've": "it would have",
  "it'll": "it will",
  "it'll've": "it will have",
  "it's": "it is",
  "let's": "let us",
  "ma'am": "madam",
  "mayn't": "may not",
  "might've": "might have",
  "mightn't": "might not",
  "mightn't've": "might not have",
  "must've": "must have",
  "mustn't": "must not",
  "mustn't've": "must not have",
  "needn't": "need not",
  "needn't've": "need not have",
  "o'clock": "of the clock",
  "oughtn't": "ought not",
  "oughtn't've": "ought not have",
  "shan't": "shall not",
  "sha'n't": "shall not",
  "shan't've": "shall not have",
  "she'd": "she would",
  "she'd've": "she would have",
  "she'll": "she will",
  "she'll've": "she will have",
  "she's": "she is",
  "should've": "should have",
  "shouldn't": "should not",
  "shouldn't've": "should not have",
  "so've": "so have",
  "so's": "so is",
  "that'd": "that would",
  "that'd've": "that would have",
  "that's": "that is",
  "there'd": "there had",
  "there'd've": "there would have",
  "there's": "there is",
  "they'd": "they would",
  "they'd've": "they would have",
  "they'll": "they will",
  "they'll've": "they will have",
  "they're": "they are",
  "they've": "they have",
  "to've": "to have",
  "wasn't": "was not",
  "we'd": "we had",
  "we'd've": "we would have",
  "we'll": "we will",
  "we'll've": "we will have",
  "we're": "we are",
  "we've": "we have",
  "weren't": "were not",
  "what'll": "what will",
  "what'll've": "what will have",
  "what're": "what are",
  "what's": "what is",
  "what've": "what have",
  "when's": "when is",
  "when've": "when have",
  "where'd": "where did",
  "where's": "where is",
  "where've": "where have",
  "who'll": "who will",
  "who'll've": "who will have",
  "who's": "who is",
  "who've": "who have",
  "why's": "why is",
  "why've": "why have",
  "will've": "will have",
  "won't": "will not",
  "won't've": "will not have",
  "would've": "would have",
  "wouldn't": "would not",
  "wouldn't've": "would not have",
  "y'all": "you all",
  "y'alls": "you alls",
  "y'all'd": "you all would",
  "y'all'd've": "you all would have",
  "y'all're": "you all are",
  "y'all've": "you all have",
  "you'd": "you had",
  "you'd've": "you would have",
  "you'll": "you you will",
  "you'll've": "you you will have",
  "you're": "you are",
  "you've": "you have"
}

FUNCTORS_WORDLIST = ["a","about", "above", "across", "after", "afterwards", "again", "against", "all", "almost", "alone", "along", "already", "also", "although", "always", "am", "among", "amongst", "amoungst", "an", "and", "another", "any", "anyhow", "anyone", "anything", "anyway", "anywhere", "are", "around", "as", "at", "be", "became", "because", "been", "before", "beforehand", "behind", "being", "below", "beside", "besides", "between", "beyond", "both", "but", "by", "can", "cannot", "could", "dare", "despite", "did", "do", "does", "done", "down", "during", "each", "eg", "either", "else", "elsewhere", "enough", "etc", "even", "ever", "every", "everyone", "everything", "everywhere", "except", "few", "first", "for", "former", "formerly", "from", "further", "furthermore", "had", "has", "have", "he", "hence", "her", "here", "hereabouts", "hereafter", "hereby", "herein", "hereinafter", "heretofore", "hereunder", "hereupon", "herewith", "hers", "herself", "him", "himself", "his", "how", "however", "i", "ie", "if", "in", "indeed", "inside", "instead", "into", "is", "it", "its", "itself", "last", "latter", "latterly", "least", "less", "lot", "lots", "many", "may", "me", "meanwhile", "might", "mine", "more", "moreover", "most", "mostly", "much", "must", "my", "myself", "namely", "near", "need", "neither", "never", "nevertheless", "next", "no", "nobody", "none", "noone", "nor", "not", "nothing", "now", "nowhere", "of", "off", "often", "oftentimes", "on", "once", "one", "only", "onto", "or", "other", "others", "otherwise", "ought", "our", "ours", "ourselves", "out", "outside", "over", "per", "perhaps", "rather", "re", "same", "second", "several", "shall", "she", "should", "since", "so", "some", "somehow", "someone", "something", "sometime", "sometimes", "somewhat", "somewhere", "still", "such", "than", "that", "the", "their", "theirs", "them", "themselves", "then", "thence", "there", "thereabouts", "thereafter", "thereby", "therefore", "therein", "thereof", "thereon", "thereupon", "these", "they", "third", "this", "those", "though", "through", "throughout", "thru", "thus", "to", "together", "too", "top", "toward", "towards", "under", "until", "up", "upon", "us", "used", "very", "via", "was", "we", "well", "were", "what", "whatever", "when", "whence", "whenever", "where", "whereafter", "whereas", "whereby", "wherein", "whereupon", "wherever", "whether", "which", "while", "whither", "who", "whoever", "whole", "whom", "whose", "why", "whyever", "will", "with", "within", "without", "would", "yes", "yet", "you", "your", "yours", "yourself", "yourselves"]

# STOPWORDS LIST FOR LANGUAGES
STOPWORDS_en = {'a', 'about', 'above', 'across', 'after', 'afterwards', 'again', 'against', 'ain', 'all', 'almost', 'alone', 'along', 'already', 'also', 'although', 'always', 'am', 'among', 'amongst', 'amount', 'an', 'and', 'another', 'any', 'anyhow', 'anyone', 'anything', 'anyway', 'anywhere', 'are', 'aren', "aren't", 'around', 'as', 'at', 'back', 'be', 'became', 'because', 'become', 'becomes', 'becoming', 'been', 'before', 'beforehand', 'behind', 'being', 'below', 'beside', 'besides', 'between', 'beyond', 'both', 'bottom', 'but', 'by', 'ca', 'call', 'can', 'cannot', 'could', 'couldn', "couldn't", 'd', 'did', 'didn', "didn't", 'do', 'does', 'doesn', "doesn't", 'doing', 'don', "don't", 'done', 'down', 'due', 'during', 'each', 'eight', 'either', 'eleven', 'else', 'elsewhere', 'empty', 'enough', 'even', 'ever', 'every', 'everyone', 'everything', 'everywhere', 'except', 'few', 'fifteen', 'fifty', 'first', 'five', 'for', 'former', 'formerly', 'forty', 'four', 'from', 'front', 'full', 'further', 'get', 'give', 'go', 'had', 'hadn', "hadn't", 'has', 'hasn', "hasn't", 'have', 'haven', "haven't", 'having', 'he', 'hence', 'her', 'here', 'hereafter', 'hereby', 'herein', 'hereupon', 'hers', 'herself', 'him', 'himself', 'his', 'how', 'however', 'hundred', 'i', 'if', 'in', 'indeed', 'into', 'is', 'isn', "isn't", 'it', "it's", 'its', 'itself', 'just', 'keep', 'last', 'latter', 'latterly', 'least', 'less', 'll', 'm', 'ma', 'made', 'make', 'many', 'may', 'me', 'meanwhile', 'might', 'mightn', "mightn't", 'mine', 'more', 'moreover', 'most', 'mostly', 'move', 'much', 'must', 'mustn', "mustn't", 'my', 'myself', 'name', 'namely', 'needn', "needn't", 'neither', 'never', 'nevertheless', 'next', 'nine', 'no', 'nobody', 'none', 'noone', 'nor', 'not', 'nothing', 'now', 'nowhere', 'o', 'of', 'off', 'often', 'on', 'once', 'one', 'only', 'onto', 'or', 'other', 'others', 'otherwise', 'our', 'ours', 'ourselves', 'out', 'over', 'own', 'part', 'per', 'perhaps', 'please', 'put', 'quite', 'rather', 're', 'really', 'regarding', 's', 'same', 'say', 'see', 'seem', 'seemed', 'seeming', 'seems', 'serious', 'several', 'shan', "shan't", 'she', "she's", 'should', "should've", 'shouldn', "shouldn't", 'show', 'side', 'since', 'six', 'sixty', 'so', 'some', 'somehow', 'someone', 'something', 'sometime', 'sometimes', 'somewhere', 'still', 'such', 't', 'take', 'ten', 'than', 'that', "that'll", 'the', 'their', 'theirs', 'them', 'themselves', 'then', 'thence', 'there', 'thereafter', 'thereby', 'therefore', 'therein', 'thereupon', 'these', 'they', 'third', 'this', 'those', 'though', 'three', 'through', 'throughout', 'thru', 'thus', 'to', 'together', 'too', 'top', 'toward', 'towards', 'twelve', 'twenty', 'two', 'under', 'unless', 'until', 'up', 'upon', 'us', 'used', 'using', 'various', 've', 'very', 'via', 'was', 'wasn', "wasn't", 'we', 'well', 'were', 'weren', "weren't", 'what', 'whatever', 'when', 'whence', 'whenever', 'where', 'whereafter', 'whereas', 'whereby', 'wherein', 'whereupon', 'wherever', 'whether', 'which', 'while', 'whither', 'who', 'whoever', 'whole', 'whom', 'whose', 'why', 'will', 'with', 'within', 'without', 'won', "won't", 'would', 'wouldn', "wouldn't", 'y', 'yet', 'you', "you'd", "you'll", "you're", "you've", 'your', 'yours', 'yourself', 'yourselves'}
STOPWORDS_es = {'agregó', 'poder', 'debe', 'cerca', 'intentais', 'está', 'todas', 'durante', 'pesar', 'estoy', 'ello', 'buenas', 'hacemos', 'adelante', 'mismas', 'trabajo', 'quién', 'desde', 'cierta', 'suyas', 'próximos', 'ahí', 'ningunos', 'casi', 'dicho', 'cual', 'ex', 'vais', 'empleais', 'despues', 'siendo', 'general', 'sean', 'otras', 'deprisa', 'dónde', 'esta', 'aún', 'nueva', 'cuántos', 'intenta', 'porque', 'bueno', 'podría', 'vuestras', 'debido', 'través', 'enseguida', 'lleva', 'sé', 'alrededor', 'horas', 'trabajar', 'mía', 'hacen', 'medio', 'algún', 'mayor', 'habrá', 'conseguimos', 'mis', 'tendrá', 'mios', 'aproximadamente', 'eramos', 'buen', 'además', 'emplean', 'quiere', 'conocer', 'expresó', 'ti', 'empleo', 'buenos', 'cuanto', 'sus', 'no', 'tras', 'raras', 'ante', 'cuándo', 'posible', 'estan', 'vaya', 'vuestro', 'éstas', 'arribaabajo', 'arriba', 'trabajas', 'cualquier', 'mismo', 'salvo', 'sino', 'este', 'contra', 'delante', 'soyos', 'asi', 'de', 'dentro', 'informó', 'podriamos', 'sabemos', 'trabaja', 'hacer', 'eres', 'aquellas', 'partir', 'otros', 'quienes', 'cuánta', 'mediante', 'qué', 'empleas', 'haces', 'aquí', 'todo', 'parece', 'dicen', 'ser', 'vuestros', 'somos', 'tuvo', 'total', 'nos', 'va', 'quiza', 'segundo', 'si', 'cuánto', 'luego', 'estar', 'ampleamos', 'hacia', 'mia', 'informo', 'también', 'existe', 'igual', 'eso', 'te', 'ella', 'nadie', 'estas', 'ningunas', 'largo', 'consideró', 'llegó', 'quizás', 'momento', 'diferentes', 'ademas', 'nuestros', 'llevar', 'detras', 'dieron', 'propio', 'ese', 'sois', 'cuáles', 'ciertos', 'suya', 'alguna', 'manifestó', 'esa', 'veces', 'actualmente', 'todavía', 'muchas', 'tengo', 'tan', 'serán', 'hizo', 'nuevo', 'gran', 'se', 'intentamos', 'es', 'demás', 'cuantas', 'vosotros', 'aquellos', 'tiempo', 'tuyos', 'lugar', 'cuatro', 'estamos', 'toda', 'alguno', 'como', 'hago', 'propios', 'mal', 'ocho', 'tres', 'diferente', 'el', 'mí', 'adrede', 'breve', 'hemos', 'fueron', 'nosotros', 'verdad', 'nunca', 'sobre', 'pasada', 'después', 'quizá', 'con', 'intento', 'podria', 'contigo', 'mencionó', 'quedó', 'modo', 'misma', 'apenas', 'ha', 'encima', 'consigue', 'nuevos', 'pudo', 'poca', 'consigues', 'otra', 'bajo', 'aquello', 'saben', 'hacerlo', 'tuya', 'fuera', 'dia', 'cuando', 'dio', 'mejor', 'ellas', 'tenga', 'al', 'tambien', 'conseguir', 'podemos', 'estados', 'hicieron', 'tenía', 'hablan', 'despacio', 'según', 'junto', 'así', 'lejos', 'último', 'conmigo', 'habia', 'pocos', 'antaño', 'cierto', 'vamos', 'incluso', 'día', 'queremos', 'temprano', 'demasiado', 'más', 'supuesto', 'muchos', 'primera', 'algunas', 'propia', 'debajo', 'ninguno', 'tú', 'estado', 'mismos', 'pero', 'proximo', 'usas', 'esos', 'podrán', 'muy', 'repente', 'trabajan', 'ya', 'usamos', 'emplear', 'nuevas', 'dos', 'ésas', 'intentas', 'cuanta', 'soy', 'tercera', 'quizas', 'ver', 'hasta', 'considera', 'las', 'cuenta', 'alli', 'aquéllos', 'aquéllas', 'decir', 'ahi', 'usais', 'paìs', 'sabes', 'próximo', 'ultimo', 'usan', 'lado', 'indicó', 'pronto', 'éstos', 'podrían', 'aquél', 'primeros', 'cuántas', 'sigue', 'ahora', 'intentan', 'ningún', 'os', 'teneis', 'añadió', 'cuál', 'estos', 'primer', 'unas', 'estaba', 'excepto', 'bien', 'antano', 'solamente', 'aseguró', 'pocas', 'buena', 'tanto', 'cuantos', 'afirmó', 'podriais', 'por', 'pues', 'aqui', 'yo', 'acuerdo', 'ellos', 'él', 'dado', 'menos', 'comentó', 'son', 'estará', 'tal', 'tenemos', 'tuyas', 'le', 'aquélla', 'unos', 'atras', 'realizado', 'respecto', 'tendrán', 'nada', 'consiguen', 'trabajais', 'eras', 'sera', 'explicó', 'intentar', 'señaló', 'cómo', 'cinco', 'bastante', 'esas', 'consigo', 'segunda', 'haber', 'solos', 'seis', 'propias', 'ayer', 'final', 'había', 'sabe', 'sí', 'claro', 'tienen', 'mío', 'usa', 'varias', 'fue', 'verdadero', 'ésos', 'mucho', 'mi', 'fui', 'realizó', 'algo', 'era', 'antes', 'nosotras', 'para', 'hay', 'haceis', 'sólo', 'sin', 'sola', 'enfrente', 'cuales', 'grandes', 'principalmente', 'tus', 'vez', 'sea', 'qeu', 'segun', 'peor', 'un', 'poco', 'la', 'manera', 'mucha', 'embargo', 'puedo', 'encuentra', 'fin', 'detrás', 'gueno', 'su', 'anterior', 'da', 'ir', 'tampoco', 'tuyo', 'vosotras', 'en', 'lo', 'todavia', 'han', 'parte', 'solo', 'pasado', 'dar', 'allí', 'aunque', 'saber', 'creo', 'podeis', 'últimas', 'días', 'donde', 'deben', 'tarde', 'esto', 'estaban', 'ése', 'sería', 'dias', 'cosas', 'aquella', 'mas', 'ni', 'fuimos', 'voy', 'pueda', 'aun', 'mientras', 'pueden', 'tu', 'tenido', 'solas', 'tener', 'cada', 'última', 'habían', 'entonces', 'todos', 'siguiente', 'haciendo', 'tiene', 'últimos', 'mías', 'ninguna', 'trabajamos', 'ejemplo', 'sabeis', 'ambos', 'ciertas', 'dijo', 'hubo', 'usted', 'podrá', 'los', 'mio', 'nuestro', 'hace', 'les', 'usar', 'éste', 'menudo', 'aquel', 'realizar', 'primero', 'hecho', 'dijeron', 'haya', 'habla', 'dan', 'siempre', 'quiénes', 'eran', 'trata', 'uso', 'nuestra', 'podrias', 'ustedes', 'quien', 'he', 'dejó', 'varios', 'míos', 'pais', 'mias', 'del', 'poner', 'estuvo', 'nuestras', 'están', 'que', 'existen', 'vuestra', 'hoy', 'me', 'suyo', 'ésta', 'valor', 'será', 'entre', 'algunos', 'podrian', 'puede', 'siete', 'van', 'dice', 'ésa', 'sido', 'otro', 'uno', 'verdadera', 'estais', 'una'}
STOPWORDS_fr = {'allo', 'derriere', 'onzième', 'oust', 'vôtres', 'durant', 'troisièmement', 'rendre', 'serait', 'olé', 'sept', 'comme', 'souvent', 'suivants', 'soit', 'desormais', 'hou', 'ore', 'quand', 'lesquelles', 'boum', 'toi', 'prealable', 'dix-huit', 'sixième', 'trois', 'ceux-ci', 'derrière', 'différentes', 'hi', 'ouf', 'houp', 'fait', 'o', 'etant', 'dont', 'suffisante', 'aucune', 'suivre', 'minimale', 'pendant', 'sauf', 'specifiques', 'seul', 'vais', 'suivantes', 'beau', 'd’', 'possibles', 'avait', 'leur', 'plouf', 'peux', 'quels', 'dixième', 'différent', 'huitième', 'puis', 'vlan', 'relativement', 'anterieures', 'était', 'bravo', 'tiens', 'ayant', 'seulement', 'quelle', 'pfft', 'suivant', 'particulièrement', 'quatorze', 'pouah', 'deuxièmement', 'seize', 'ohé', 'quinze', 'autrui', 'alors', 'tien', 'probable', 'divers', 'avons', 'parfois', 'pan', 'dessus', 'ça', 'chez', 'allô', 'celles-là', 'notre', 'aux', 'quel', 'etc', 'pouvait', 'ô', 'troisième', 'seraient', 'apres', 'restant', 'pure', 'soi', 'hors', 'memes', 'de', 'auraient', 'rare', 'soixante', 'sacrebleu', 'celui-ci', 'mon', 'ho', 'subtiles', 's’', 'hep', "qu'", 'reste', 'on', 'celui-là', 'pas', 'quiconque', 'toutes', 'par', 'deja', 'naturelle', 'specifique', 'nos', 'va', 'si', 'malgre', 'doit', 'lesquels', 'quatre-vingt', 'tel', 'ces', 'depuis', 'onze', 'brrr', 'ce', 'tous', 'aie', 'lui', 'te', 'celui', 'moi', 'douze', 'cependant', 'zut', 'mes', 'au', 'laquelle', 'gens', 'différente', 'voici', 'là', 'sien', 'parle', 'nôtres', 'c’', 'toujours', 'quatre', 'restrictif', 'basee', 'trop', 'très', 'peu', 'tes', 'huit', 'chaque', 'procedant', 'superpose', 'tsoin', 'avec', 'speculatif', 'vont', 'parce', 'pense', 'voilà', 'telle', 'outre', 'se', 'vous', 'eux', 'es', 'désormais', 'etais', 'personne', 'elle', 'ceci', 'proche', 'passé', 'quoique', 'sont', 'siennes', 'derniere', 'aura', 'clac', "m'", 'néanmoins', 'pire', 'quant-à-soi', 'rares', 'flac', 'vu', 'tres', 'meme', 'uns', 'vif', 'laisser', "l'", "t'", 'ont', 'hue', 'ès', 'auquel', 'suivante', 'assez', 'quelques', 'egales', 'fi', 'vas', 'nombreuses', 'etre', 'sent', 'crac', 'ses', 'clic', 'precisement', 'ha', 'necessairement', 'psitt', 'importe', 'premièrement', 'debout', 'encore', 'malgré', 'font', 'ouverts', 'vous-mêmes', 'remarquable', 'auront', 'ai', 'tend', 'possessifs', 'couic', 'quant', 'exterieur', 'tels', 'plus', 'toi-même', 'elles-mêmes', 'avaient', 'moins', 'pu', 'sait', 'tente', 'suffit', 'sienne', 'trente', 'suis', 'toc', 'tenir', 'feront', 'ouverte', 'ailleurs', 'enfin', 'nous-mêmes', 'tellement', 'allaient', 'bah', 'possible', 'abord', 'pour', 'eux-mêmes', 'elle-même', 'neuvième', 'floc', 'dring', 'stop', 'celles', 'moindres', "c'", 'sa', 'vé', 'hui', 'uniques', 'moi-meme', 'afin', 'celles-ci', 'exactement', 'faisant', 'lès', 'neanmoins', 'semblable', 'et', 'treize', 'allons', 'probante', 'las', 'tardive', 'aucun', 'uniformement', 'nous', 'ou', 'retour', 'vives', 'celle-là', 'differentes', 'moyennant', 'tout', 'dix', 'devant', 'vive', 'cette', 'cet', 'dite', 'bien', 'lorsque', 'absolument', 'quelconque', 'juste', 'hum', 'hop', 'directe', 'ait', 'different', 'rend', 'je', 'chacune', 'â', 'ceux', 'nombreux', 'na', 'après', 'cinquante', 'egale', 'peuvent', 'surtout', 'hé', 'comment', 'son', 'chère', 'nul', 'avais', 'ta', 'auxquels', 'effet', 'il', 'votre', 'le', 'sein', 'avant', 'certaines', 'ma', 'est', 't’', 'ceux-là', 'notamment', 'sera', 'delà', 'près', 'sans', 'diverse', 'pourrais', 'mien', 'vifs', 'siens', 'vivat', 'lui-même', 'mais', 'elles', 'première', 'qui', 'plutôt', 'autres', 'tsouin', 'touchant', 'dernier', 'plein', 'hormis', 'suffisant', 'suit', 'premier', 'ils', "n'", 'vingt', 'avoir', 'merci', 'cinq', 'eu', 'contre', 'combien', 'parlent', 'quanta', 'étais', 'restent', 'dans', 'celle', 'tienne', 'dessous', 'aussi', 'nouveau', 'unique', 'sapristi', 'anterieure', 'duquel', 'excepté', 'fais', 'bas', 'longtemps', 'neuf', 'holà', 'semblaient', 'telles', 'hurrah', 'plusieurs', 'a', 'quatrièmement', 'donc', 'differents', 'particulier', 'strictement', 'chacun', 'quarante', 'as', 'pourrait', 'certaine', 'semble', 'etaient', 'mille', 'pourquoi', 'un', 'bat', 'tic', 'maint', 'différents', 'la', 'nôtre', 'oh', 'attendu', 'lequel', 'eh', 'aujourd', 'leurs', 'paf', 'diverses', 'dix-sept', 'douzième', 'autrement', 'maintenant', "s'", 'da', 'etait', 'n’', 'moi-même', 'en', 'cinquantaine', 'desquels', 'parler', 'vers', 'certes', 'unes', 'lors', 'dit', 'dehors', 'lui-meme', 'pfut', "d'", 'façon', 'j’', 'étaient', 'tant', 'peut', 'partant', 'desquelles', 'hein', 'rien', 'possessif', 'aupres', 'auxquelles', 'envers', 'quelles', 'pur', 'via', 'toutefois', 'ni', 'ainsi', 'revoilà', 'chers', 'pres', 'parseme', 'quelque', 'tu', 'ton', 'maximale', 'toute', 'cher', 'autre', 'deux', 'étant', 'cela', 'hélas', 'celle-ci', 'chiche', 'faisaient', 'environ', 'pff', 'autrefois', 'té', 'des', 'devra', 'hem', 'i', 'miennes', 'chut', 'cent', 'du', 'necessaire', 'soi-même', "aujourd'hui", 'dits', 'ouste', 'relative', 'les', 'certain', 'revoici', 'concernant', 'miens', 'doivent', 'euh', 'non', 'tac', 'vôtre', 'anterieur', 'seule', 'egalement', 'm’', 'dire', 'sur', 'dès', 'selon', 'aurait', 'six', 'rarement', 'pif', 'car', 'été', 'cinquième', 'certains', 'seront', 'dix-neuf', 'même', "quelqu'un", 'être', 'bigre', 'ouvert', 'cinquantième', 'semblent', 'naturelles', 'que', 'dedans', 'beaucoup', 'directement', 'vos', 'ah', 'jusque', 'sous', 'ci', 'tenant', 'une', 'l’', 'ouias', 'mêmes', 'parmi', 'me', 'septième', 'quoi', 'naturel', 'devers', 'où', 'tiennes', 'qu’', 'deuxième', 'particulière', 'extenso', 'à', 'chères', 'comparables', 'entre', 'mince', 'ne', 'permet', 'quatrième', 'mienne', 'puisque', 'jusqu', 'ollé', "j'", 'sinon', 'compris', 'comparable'}
STOPWORDS_ru = {'моего', 'все', 'нашей', 'такая', 'нею', 'его', 'ем', 'от', 'моя', 'свою', 'своих', 'бы', 'будучи', 'нет', 'нему', 'сами', 'ела', 'это', 'когда', 'одним', 'кого', 'которою', 'ней', 'одному', 'свое', 'этою', 'нашею', 'моими', 'мое', 'всей', 'одних', 'мочь', 'ест', 'они', 'который', 'наш', 'тою', 'он', 'кто', 'всю', 'чтобы', 'наше', 'такого', 'нашим', 'всея', 'из', 'которого', 'эти', 'по', 'те', 'них', 'едим', 'мой', 'буду', 'может', 'на', 'ей', 'мы', 'есть', 'одни', 'самим', 'свой', 'которым', 'таком', 'этих', 'мне', 'могут', 'всеми', 'нее', 'она', 'такой', 'для', 'такую', 'теми', 'вот', 'которой', 'сама', 'саму', 'её', 'наши', 'которыми', 'что', 'оно', 'с', 'одну', 'этот', 'будете', 'могу', 'себе', 'такие', 'ними', 'которую', 'тех', 'могла', 'самом', 'нам', 'одно', 'тот', 'ему', 'всем', 'этими', 'но', 'своей', 'во', 'будем', 'всём', 'вами', 'своем', 'сам', 'эту', 'их', 'тобою', 'том', 'ею', 'еще', 'одна', 'своего', 'и', 'был', 'будь', 'всех', 'комья', 'как', 'такому', 'самими', 'можешь', 'можете', 'одними', 'только', 'чего', 'моё', 'моей', 'свои', 'своя', 'тобой', 'той', 'будешь', 'один', 'моим', 'мог', 'своею', 'же', 'ты', 'или', 'этому', 'могли', 'могло', 'своим', 'ел', 'моею', 'собой', 'наших', 'которые', 'вся', 'моему', 'мою', 'не', 'мои', 'я', 'ее', 'уже', 'да', 'одного', 'этой', 'была', 'нашем', 'можем', 'а', 'этим', 'которому', 'само', 'эта', 'за', 'себя', 'этом', 'кому', 'нашему', 'будьте', 'чему', 'наса', 'неё', 'одном', 'которое', 'томах', 'него', 'того', 'вам', 'емъ', 'тому', 'нами', 'которая', 'таким', 'тем', 'ко', 'у', 'ещё', 'самих', 'в', 'так', 'такое', 'своём', 'меня', 'самого', 'всему', 'будет', 'ими', 'моём', 'при', 'собою', 'одной', 'ешь', 'та', 'тебя', 'чем', 'ним', 'оне', 'вы', 'всего', 'к', 'об', 'будут', 'ком', 'если', 'наша', 'ту', 'были', 'было', 'моем', 'едят', 'могите', 'всею', 'своими', 'своему', 'до', 'то', 'нашими', 'нашего', 'такою', 'нем', 'им', 'которых', 'нас', 'имъ', 'мною', 'одною', 'самому', 'вас', 'кем', 'мной', 'нём', 'всё', 'тебе', 'котором', 'моги', 'такими', 'чём', 'моих', 'весь', 'нашу', 'быть', 'о', 'своё', 'таких', 'этого'}
STOPWORDS_yo = {'fún', 'è', 'ní', 'rẹ̀', 'k', 'sínú', 'àwọn', 'n', 'j', 'á', 'ti', 'ọ', 'mo', 'mọ̀', 'ù', 'ì', 'kì', 'tí', 'ṣùgbọ́n', 'p', 'máa', 'nítorí', 'nígbà', 'jẹ', 'u', 'm', 'púpọ̀', 'í', 'padà', 'jẹ́', 'náà', 'bí', 'bá', 'wọ́n', 'ẹ', 'wọn', 'ni', 'àti', 'b', 'gbogbo', 'yìí', 'ò', 'ṣé', '́', 't', 'f', 'láti', '̣', 'ẹmọ́', 'bẹ̀rẹ̀', 'ọ̀pọ̀lọpọ̀', 'w', '̀', 'mi', 'ǹ', 'sí', 'd', 'r', 'pẹ̀lú', 'wà', 'inú', 'ó', 'òun', 'wá', 'à', 'y', 'kò', 'fẹ́', 'i', 'e', 'ńlá', 'sì', 'nǹkan', 'lè', 'kan', 'jù', 'l', 'pé', 'é', 's', 'o', 'ọjọ́', 'g', 'ń', 'ṣe', 'an', 'a', 'lọ', 'ú', 'kí', 'ṣ'}
STOPWORDS_de = {'dieselbe', 'nahm', 'drei', 'eigen', 'vielen', 'guter', 'nun', 'würden', 'zu', 'sollten', 'derselben', 'vergangenen', 'dasselbe', 'grosse', 'könnte', 'á', 'wegen', 'bereits', 'demgegenüber', 'acht', 'mögen', 'meinem', 'rechter', 'jedermanns', 'weiteres', 'dir', 'trotzdem', 'wohl', 'wird', 'des', 'durchaus', 'her', 'je', 'außer', 'machte', 'infolgedessen', 'erst', 'später', 'während', 'derselbe', 'unsere', 'bisher', 'wir', 'großer', 'allen', 'dürft', 'ihre', 'hätten', 'in', 'jahr', 'wenn', 'dermassen', 'ihrem', 'gibt', 'recht', 'erstes', 'schon', 'überhaupt', 'sei', 'eigenen', 'achte', 'hat', 'jemand', 'eines', 'ausser', 'sonst', 'hoch', 'siebentes', 'immer', 'unter', 'gekonnt', 'denn', 'habe', 'hast', 'jenes', 'einmal', 'alles', 'sollte', 'lange', 'jeder', 'weniges', 'gar', 'wem', 'seine', 'fünfte', 'währenddessen', 'elf', 'gegen', 'dahin', 'wenigstens', 'besser', 'zwar', 'weiteren', 'allein', 'seitdem', 'kein', 'sechstes', 'vier', 'damals', 'zehnten', 'endlich', 'der', 'früher', 'wurde', 'war', 'mögt', 'los', 'rechte', 'zunächst', 'dieses', 'dasein', 'ganzen', 'siebte', 'nur', 'gemusst', 'ihm', 'jedermann', 'große', 'drin', 'jetzt', 'eine', 'danach', 'solchem', 'mussten', 'doch', 'siebtes', 'niemanden', 'ausserdem', 'etwas', 'davon', 'dementsprechend', 'da', 'hatten', 'dem', 'am', 'jemanden', 'jahren', 'hier', 'er', 'sollen', 'sowie', 'kann', 'tag', 'nie', 'deren', 'auf', 'magst', 'hatte', 'zusammen', 'konnten', 'zehn', 'ganz', 'diesem', 'desselben', 'statt', 'nachdem', 'gab', 'meines', 'seien', 'einigen', 'neunter', 'zehntes', 'weitere', 'im', 'manches', 'würde', 'hin', 'bald', 'als', 'daneben', 'lang', 'kleiner', 'willst', 'sieben', 'mag', 'ach', 'gleich', 'daß', 'deinem', 'selbst', 'gesagt', 'siebente', 'zwei', 'darfst', 'möchte', 'beide', 'dies', 'denselben', 'worden', 'ihn', 'allgemeinen', 'das', 'wie', 'zwanzig', 'sind', 'zeit', 'demzufolge', 'demgemäss', 'groß', 'kleine', 'uhr', 'fünftes', 'gut', 'nicht', 'meinen', 'so', 'nein', 'ihres', 'welcher', 'ihren', 'zweites', 'weniger', 'manchen', 'darf', 'dann', 'einige', 'allem', 'ab', 'ist', 'anderen', 'bekannt', 'möglich', 'werde', 'meiner', 'dabei', 'sein', 'mir', 'dank', 'dein', 'diejenige', 'soll', 'jeden', 'manchem', 'kam', 'jener', 'dafür', 'heißt', 'dieselben', 'beim', 'dich', 'leicht', 'weit', 'solang', 'aller', 'zur', 'über', 'demselben', 'alle', 'ihr', 'ersten', 'viele', 'jene', 'en', 'einen', 'beispiel', 'durfte', 'noch', 'rechten', 'dessen', 'keinen', 'tat', 'ganzes', 'dagegen', 'wessen', 'einander', 'seinem', 'kommen', 'richtig', 'neuntes', 'eigener', 'seinen', 'großen', 'ihrer', 'a', 'darunter', 'darauf', 'ehrlich', 'mittel', 'bin', 'waren', 'was', 'übrigens', 'dort', 'vom', 'wollte', 'darum', 'vierten', 'dritter', 'drittes', 'macht', 'wenige', 'unserer', 'genug', 'einiges', 'neunten', 'bist', 'mochte', 'dahinter', 'ein', 'siebenten', 'unser', 'allerdings', 'daran', 'dritte', 'gerade', 'sehr', 'muß', 'dazu', 'den', 'dieser', 'teil', 'muss', 'erster', 'damit', 'neben', 'neue', 'mich', 'oben', 'sie', 'kannst', 'gewesen', 'keiner', 'könnt', 'keinem', 'nach', 'mehr', 'will', 'musste', 'einiger', 'deiner', 'jedem', 'natürlich', 'eigenes', 'ende', 'schlecht', 'achten', 'werden', 'zweiter', 'jedoch', 'sechste', 'sechster', 'jahre', 'ich', 'tel', 'wann', 'zweite', 'vergangene', 'gutes', 'neuen', 'seines', 'die', 'gekannt', 'wollt', 'offen', 'heisst', 'ins', 'rechtes', 'dadurch', 'satt', 'ihnen', 'solches', 'zehnter', 'kurz', 'grosses', 'von', 'können', 'seiner', 'na', 'niemandem', 'zehnte', 'uns', 'anderem', 'einem', 'solchen', 'hinter', 'währenddem', 'geworden', 'rund', 'jemandem', 'manche', 'einmaleins', 'grossen', 'durften', 'siebter', 'welche', 'siebten', 'diejenigen', 'denen', 'achter', 'müsst', 'weil', 'diesen', 'deswegen', 'kleines', 'sechsten', 'solche', 'ganze', 'müssen', 'nichts', 'ag', 'derjenigen', 'fünfter', 'mochten', 'hätte', 'konnte', 'sondern', 'wahr', 'vielleicht', 'auch', 'sich', 'wollten', 'indem', 'dürfen', 'grosser', 'lieber', 'haben', 'etwa', 'daher', 'daraus', 'darüber', 'eben', 'davor', 'ohne', 'zweiten', 'fünf', 'mancher', 'zuerst', 'wo', 'außerdem', 'jenem', 'sagte', 'man', 'andere', 'neun', 'zurück', 'gemocht', 'welches', 'oder', 'gehabt', 'gewollt', 'heute', 'um', 'geht', 'geschweige', 'niemand', 'dazwischen', 'dermaßen', 'also', 'musst', 'wurden', 'jenen', 'demgemäß', 'keine', 'wer', 'durch', 'tage', 'daselbst', 'deshalb', 'wirklich', 'mein', 'darin', 'vielem', 'viel', 'bis', 'euch', 'vierte', 'entweder', 'vor', 'gedurft', 'ja', 'morgen', 'dritten', 'werdet', 'siebenter', 'du', 'leider', 'irgend', 'seit', 'machen', 'wollen', 'besonders', 'zum', 'ob', 'zwischen', 'warum', 'wart', 'ganzer', 'gegenüber', 'kaum', 'tun', 'andern', 'wäre', 'achtes', 'aber', 'ging', 'einer', 'besten', 'wenig', 'kommt', 'gute', 'weiter', 'und', 'großes', 'vierter', 'erste', 'gemacht', 'mit', 'fünften', 'für', 'wirst', 'kleinen', 'viertes', 'jede', 'meine', 'aus', 'solcher', 'gern', 'diese', 'welchem', 'oft', 'beiden', 'habt', 'sechs', 'zugleich', 'sah', 'bei', 'gross', 'es', 'tagen', 'dass', 'sagt', 'seid', 'wen', 'eigene', 'derjenige', 'neunte', 'ebenso', 'an', 'deine', 'anders', 'wieder', 'welchen', 'gehen'}



__version__ = '0.1.1'
from .neattext import TextCleaner
from .neattext import TextExtractor
from .neattext import TextMetrics
from .neattext import TextFrame

# import neattext.functions ##worked
from neattext.functions import *
from neattext.pattern_data import *
from neattext.explainer import *






# -*- coding: utf-8 -*-
# This file is part of the NEAT Project suite of libraries

import re
from neattext.pattern_data import (
    EMAIL_REGEX,
    NUMBERS_REGEX,
    PHONE_REGEX,
    SPECIAL_CHARACTERS_REGEX,
    EMOJI_REGEX,
    URL_PATTERN,
    CURRENCY_REGEX,
    CURRENCY_SYMB_REGEX,
    STOPWORDS,
    DATE_REGEX,
    MOST_COMMON_PUNCT_REGEX,
    STOPWORDS_en,
    STOPWORDS_fr,
    STOPWORDS_es,
    STOPWORDS_ru,
    STOPWORDS_yo,
    STOPWORDS_de,
    HASTAG_REGEX,
    USER_HANDLES_REGEX,
    MASTERCard_REGEX,
    VISACard_REGEX,
    BTC_ADDRESS_REGEX,
    STREET_ADDRESS_REGEX,
    PoBOX_REGEX,
    MD5_SHA_REGEX,
)

from neattext import TextFrame
from collections import defaultdict, Counter
from heapq import nlargest
import random, string, math


# Individual Functions
def remove_emails(text):
    """Returns A String with the emails removed """
    result = re.sub(EMAIL_REGEX, "", text)
    return result


def remove_numbers(text):
    """Returns A String with the numbers/digits removed """
    result = re.sub(NUMBERS_REGEX, "", text)
    return result


def remove_phone_numbers(text):
    """Returns A String with the phone numbers removed """
    result = re.sub(PHONE_REGEX, "", text)
    return result


def remove_punctuations(text, most_common=True):
    """Returns A String with punctuations remove
    Params:
            most_common:boolen(True/False)
            use the most common punctuations eg (,.?!&-_`"')

    """
    PUNCT_REGEX = re.compile(r"""[!"&'()*,-./:;?@[\\]^_`{|}]""")
    MOST_COMMON_PUNCT_REGEX = re.compile(r"""[!"&',-.;?_`]""")
    if most_common == True:
        result = re.sub(MOST_COMMON_PUNCT_REGEX, "", text)
    else:
        result = re.sub(PUNCT_REGEX, "", text)
    return result


def remove_puncts(text, most_common=True):
    """Returns A String with punctuations remove
    Params:
            most_common:boolen(True/False)
            use the most common punctuations eg (,.?!&-_`"')

    """
    if most_common:
        result = re.sub(MOST_COMMON_PUNCT_REGEX, "", text)
    else:
        result = str(text).translate(str.maketrans("", "", string.punctuation))
    return result


def remove_special_characters(text):
    """Returns A String with the specified characters removed """
    result = re.sub(SPECIAL_CHARACTERS_REGEX, "", text)
    return result


def remove_emojis(text):
    """Returns A String with the emojis removed """
    result = re.sub(EMOJI_REGEX, "", text)
    return result


def remove_stopwords(text, lang="en"):
    """Returns A String with the stopwords removed

    lang: specify language to use [en|es|fr|ru|yo|de]
            Support English(en),Spanish(es),French(fr)|Russian(ru)|Yoruba(yo)|German(de)

    """
    if lang == "en":
        stopwords_in_use = STOPWORDS_en
    elif lang == "es":
        stopwords_in_use = STOPWORDS_es
    elif lang == "fr":
        stopwords_in_use = STOPWORDS_fr
    elif lang == "ru":
        stopwords_in_use = STOPWORDS_ru
    elif lang == "yo":
        stopwords_in_use = STOPWORDS_yo
    elif lang == "de":
        stopwords_in_use = STOPWORDS_de
    else:
        stopwords_in_use = STOPWORDS_en

    result = [word for word in text.split() if word.lower() not in stopwords_in_use]
    return " ".join(result)


def remove_urls(text):
    """Returns A String with URLS removed """
    result = re.sub(URL_PATTERN, "", text)
    return result


def remove_currencies(text):
    """Returns A String with Currencies removed """
    result = re.sub(CURRENCY_REGEX, "", text)
    return result


def remove_currency_symbols(text):
    """Returns A String with Currency Symbols removed """
    result = re.sub(CURRENCY_SYMB_REGEX, "", text)
    return result


def remove_html_tags(text):
    """Returns A String with HTML Tags removed"""
    result = re.sub("<[^<]+?>", "", text)
    return result


def remove_dates(text):
    """Returns A String with Dates Removed """
    result = re.sub(DATE_REGEX, "", text)
    return result


def remove_non_ascii(text):
    """Returns A String with Non ASCII removed"""
    import unicodedata

    result = (
        unicodedata.normalize("NFKD", text)
        .encode("ascii", "ignore")
        .decode("utf-8", "ignore")
    )
    return result


def remove_multiple_spaces(text):
    """Returns A String with Multiple Whitespaces removed"""
    result = re.sub("\\s{2,}", " ", text)
    return result


def remove_bad_quotes(text):
    """Returns a string with bad quotes removed"""
    result = re.sub("[‘’“”…]", " ", text)
    return result


def remove_custom_words(text, custom_wordlist):
    """Returns A String with the custom wordlist removed """
    result = [word for word in text.split() if word.lower() not in custom_wordlist]
    return " ".join(result)


def remove_hashtags(text):
    """Returns a string with hashtags removed"""
    result = re.sub(HASTAG_REGEX, " ", text)
    return result


def remove_userhandles(text):
    """Returns a string with @userhandles removed"""
    result = re.sub(USER_HANDLES_REGEX, " ", text)
    return result


def remove_custom_pattern(text, term_pattern):
    """Remove Custom Pattern

    Params
    ------
    text: string
    term_pattern:pattern or term to remove

    """
    result = re.sub(term_pattern, " ", text)
    return result


def remove_shortwords(text, length=3):
    """Returns a string with all short words of a particular length or less removed

    params:
    length: Specify the length of words you want to remove
    default is 3 letter word length or less

    """
    token_words = re.split(r"\W+", text)
    long_words_list = [i for i in token_words if len(i) > int(length)]
    return " ".join(long_words_list)


def remove_btc_address(text):
    """Returns a string with Bitcoin Addresses Removed removed"""
    result = re.sub(BTC_ADDRESS_REGEX, " ", text)
    return result


def remove_visacard_addr(text):
    """Returns a string with VISA Card Addresses removed"""
    result = re.sub(VISACard_REGEX, " ", text)
    return result


def remove_mastercard_addr(text):
    """Returns a string with MasterCard Addresses removed"""
    result = re.sub(MASTERCard_REGEX, " ", text)
    return result


def remove_md5sha(text):
    """Returns a string with MD5 & SHA Hash removed"""
    result = re.sub(MD5_SHA_REGEX, " ", text)
    return result


def remove_postoffice_box(text):
    """Returns a string with Post Office Box removed"""
    result = re.sub(PoBOX_REGEX, " ", text)
    return result


def remove_street_address(text):
    """Returns a string with Street Addresses removed"""
    result = re.sub(STREET_ADDRESS_REGEX, " ", text)
    return result



def remove_terms_in_bracket(text,bracket_form="curly"):
    """Returns a string with terms/words in bracket together with the bracket removed
    
    Params:
        - bracket_form: string or symbol about whether is it  curly({) or square([])
        or etc
    """
    CURLY_BRACKET_REGEX = re.compile(r"\{(.*?)\}") 
    SQUARE_BRACKET_REGEX = re.compile(r"\[(.*?)\]") 
    NORMAL_BRACKET_REGEX = re.compile(r"\((.*?)\)") 
    if bracket_form == "curly" or bracket_form == "{}":
        result = re.sub(CURLY_BRACKET_REGEX,"",text)
    elif bracket_form == "square" or bracket_form == "[]":
        result = re.sub(SQUARE_BRACKET_REGEX,"",text)
    elif bracket_form == "normal" or bracket_form == "()":
        result = re.sub(NORMAL_BRACKET_REGEX,"",text)
    return result

    # EXTRACTION FUNCTIONS


def extract_emails(text):
    """Returns the emails extracted """
    result = re.findall(EMAIL_REGEX, text)
    return result


def extract_numbers(text):
    """Returns the numbers/digits extracted """
    result = re.findall(NUMBERS_REGEX, text)
    return result


def extract_phone_numbers(text):
    """Returns the phone number extracted """
    result = re.findall(PHONE_REGEX, text)
    return result


def extract_special_characters(text):
    """Returns the specified characters extracted """
    result = re.findall(SPECIAL_CHARACTERS_REGEX, text)
    return result


def extract_emojis(text):
    """Returns the emojis extracted """
    result = re.findall(EMOJI_REGEX, text)
    return result


def extract_stopwords(text):
    """Returns the stopwords as a list """
    result = [word for word in text.split() if word in STOPWORDS]
    return result


def extract_urls(text):
    """Returns the URLS extracted """
    result = re.findall(URL_PATTERN, text)
    return result


def extract_currencies(text):
    """Returns the currencies extracted """
    result = re.findall(CURRENCY_REGEX, text)
    return result


def extract_currency_symbols(text):
    """Returns the currency symbols extracted """
    result = re.findall(CURRENCY_SYMB_REGEX, text)
    return result


def extract_dates(text):
    """Returns the dates extracted """
    result = re.findall(DATE_REGEX, text)
    return result


def extract_html_tags(text):
    """Returns  the HTML Tags extracted """
    result = re.findall(r"<[^<]+?>", text)
    return result


def extract_hashtags(text):
    """Returns the hashtags extracted"""
    result = re.findall(HASTAG_REGEX, text)
    return result


def extract_userhandles(text):
    """Returns the @userhandles extracted """
    result = re.findall(USER_HANDLES_REGEX, text)
    return result


def extract_pattern(text, term_pattern):
    """Returns  a list all terms found extracted

    --------
    text: string
    term_pattern: pattern or term to extract(you can also use r'' for raw regex pattern)

    """
    result = re.findall(term_pattern, text)
    return result


def extract_shortwords(text, length=3):
    """Returns a list with all short words of a particular length

    params:
    length: Specify the length of words you want to remove
    default is 3 letter word length

    """
    token_words = re.split(r"\W+", text)
    short_words_list = [i for i in token_words if len(i) <= int(length)]
    return short_words_list


def extract_btc_address(text):
    """Returns a list with Bitcoin Addresses """
    result = re.findall(BTC_ADDRESS_REGEX, text)
    return result


def extract_visacard_addr(text):
    """Returns a list with VISA Card Addresses """
    result = re.findall(VISACard_REGEX, text)
    return result


def extract_mastercard_addr(text):
    """Returns a list with MasterCard Addresses """
    result = re.findall(MASTERCard_REGEX, text)
    return result


def extract_md5sha(text):
    """Returns a list with MD5 & SHA Hash """
    result = re.findall(MD5_SHA_REGEX, text)
    return result


def extract_postoffice_box(text):
    """Returns a list with Post Office Box """
    result = re.findall(PoBOX_REGEX, text)
    return result


def extract_street_address(text):
    """Returns a list with Street Addresses """
    result = re.findall(STREET_ADDRESS_REGEX, text)
    return result

def extract_terms_in_bracket(text,bracket_form="curly"):
    """Returns  a list of a string within bracket 
    
    Params:
        - bracket_form: string or symbol about whether is it  curly({) or square([])
        or etc
    """
    CURLY_BRACKET_REGEX = re.compile(r"\{(.*?)\}") 
    SQUARE_BRACKET_REGEX = re.compile(r"\[(.*?)\]") 
    NORMAL_BRACKET_REGEX = re.compile(r"\((.*?)\)") 
    if bracket_form == "curly" or bracket_form == "{}":
        result = re.findall(CURLY_BRACKET_REGEX,text)
    elif bracket_form == "square" or bracket_form == "[]":
        result = re.findall(SQUARE_BRACKET_REGEX,text)
    elif bracket_form == "normal" or bracket_form == "()":
        result = re.findall(NORMAL_BRACKET_REGEX,text)

    return result


def clean_text(
    text,
    puncts=False,
    stopwords=True,
    urls=False,
    emails=False,
    numbers=False,
    emojis=True,
    special_char=False,
    phone_num=False,
    non_ascii=False,
    multiple_whitespaces=True,
    contractions=False,
    currency_symbols=False,
    custom_pattern=None,
):
    """
    Clean entire text

    >>> clean_text(mytext,puncts=True)

    Parameters
    ----------
    text

    puncts:Boolean(True/False) default is True
    remove punctuations.

    stopwords:Boolean(True/False) default is False
    remove stopwords

    urls:Boolean(True/False) default is True
    remove punctuations

    emails:Boolean(True/False) default is True
    remove emails

    emojis:Boolean(True/False) default is True
    remove emojis

    numbers:Boolean(True/False) default is False
    remove numbers

    phone_num:Boolean(True/False) default is False
    remove phone numbers

    non_ascii:Boolean(True/False) default is False
    remove non ascii characters

    multiple_whitespaces:Boolean(True/False) default is False
    remove multiple whitespaces

    contractions:Boolean(True/False) default is False
    fix contractions

    currency_symbols:Boolean(True/False) default is False
    remove currency symbols

    custom_pattern:Specify Pattern to remove from text,default is None
     example:
     >>> clean_text(mytext,custom_pattern='hello##')

    Returns
    -------
    string

    """

    if puncts:
        text = remove_punctuations(text)
    if stopwords:
        text = remove_stopwords(text)
    if emails:
        text = remove_emails(text)
    if phone_num:
        text = remove_phone_numbers(text)
    if numbers:
        text = remove_numbers(text)
    if urls:
        text = remove_urls(text)
    if emojis:
        text = remove_emojis(text)
    if non_ascii:
        text = remove_non_ascii(text)
    if multiple_whitespaces:
        text = remove_multiple_spaces(text)
    if contractions:
        text = fix_contractions(text)
    if currency_symbols:
        text = remove_currency_symbols(text)
    if special_char:
        text = remove_special_characters(text)
    if custom_pattern is not None:
        text = remove_custom_pattern(text, custom_pattern)

    return text.lower()


# def clean_text(text,preserve=False):
# 	"""Clean Entire Text"""
# 	if preserve == False:
# 		email_result = re.sub(EMAIL_REGEX,"",text)
# 		phone_result = re.sub(PHONE_REGEX,"",email_result)
# 		number_result = re.sub(NUMBERS_REGEX,"",phone_result)
# 		url_result = re.sub(URL_PATTERN,"",number_result)
# 		emoji_result = re.sub(EMOJI_REGEX,"",url_result)
# 		special_char_result = re.sub(SPECIAL_CHARACTERS_REGEX,"",emoji_result)
# 		final_result = special_char_result.lower()
# 	else:
# 		email_result = re.sub(EMAIL_REGEX,"<EMAIL>",text)
# 		phone_result = re.sub(PHONE_REGEX,"<PHONENUMBER>",email_result)
# 		number_result = re.sub(NUMBERS_REGEX,"<NUMBERS>",phone_result)
# 		url_result = re.sub(URL_PATTERN,"<URL>",number_result)
# 		emoji_result = re.sub(EMOJI_REGEX,"<EMOJI>",url_result)
# 		final_result = emoji_result.lower()

# 	return final_result


def replace_emails(text, replace_with="<EMAIL>"):
    """Replaces the emails in the text with custom label"""
    result = re.sub(EMAIL_REGEX, replace_with, text)
    return result


def replace_phone_numbers(text, replace_with="<PHONENUMBER>"):
    """Replaces the phone numbers in the text with custom label"""
    result = re.sub(PHONE_REGEX, replace_with, text)
    return result


def replace_numbers(text, replace_with="<NUMBER>"):
    """Replaces numbers/digits in the text with custom label"""
    result = re.sub(NUMBERS_REGEX, replace_with, text)
    return result


def replace_special_characters(text, replace_with="<SPECIAL_CHAR>"):
    """Replaces special characters in the text with custom label"""
    result = re.sub(SPECIAL_CHARACTERS_REGEX, replace_with, text)
    return result


def replace_emojis(text, replace_with="<EMOJI>"):
    """Replaces emojis in the text with custom label"""
    result = re.sub(EMOJI_REGEX, replace_with, text)
    return result


def replace_urls(text, replace_with="<URL>"):
    """Replaces URLS/HTTP(S) in the text with custom label"""
    result = re.sub(URL_PATTERN, replace_with, text)
    return result


def replace_currencies(text, replace_with="<CURRENCY>"):
    """Replaces Currencies in the text with custom label"""
    result = re.sub(CURRENCY_REGEX, replace_with, text)
    return result


def replace_currency_symbols(text, replace_with="<CURRENCY_SYMB>"):
    """Replaces currency symbols in the text with custom label"""
    result = re.sub(CURRENCY_SYMB_REGEX, replace_with, text)
    return result


def replace_dates(text, replace_with="<DATE>"):
    """Replaces Dates in the text with custom label"""
    result = re.sub(DATE_REGEX, replace_with, text)
    return result


def replace_term(text, old_term, new_term):
    """Replaces term in the text with another term"""
    result = re.sub(old_term, new_term, text)
    return result


def replace_bad_quotes(text):
    """Replace bad quotes with their correct once"""
    result = re.sub("[‘’“”…]", '"', text)
    return result


def read_txt(filename):
    """
    Read a Text File and Create A TextFrame From it


    Parameters
    ----------
    text : Main Text
    filename : file with text to read

    Returns
    ----------
    Returns a TextFrame for text
    """
    with open(filename, "r") as f:
        text_read = f.read()
        docx_tf = TextFrame(text_read)
    return docx_tf


def to_txt(text, filename):
    """
    Save/Write a Text  to A File


    Parameters
    ----------
    text : Main Text
    filename : file with text to write/save to

    Returns
    ----------
    Creates A New File with Text on it


    """
    with open(filename, "w") as f:
        f.write(text)


def hamming_distance(lhs, rhs):
    """Returns the Hamming Distance of Two Equal Strings

    Usage
    >>> nt.hamming_distance('Pear','Pearls')
    """
    return len([(x, y) for x, y in zip(lhs, rhs) if x != y])


__numbers_dict = {
    1: "One",
    2: "Two",
    3: "Three",
    4: "Four",
    5: "Five",
    6: "Six",
    7: "Seven",
    8: "Eight",
    9: "Nine",
    10: "Ten",
    11: "Eleven",
    12: "Twelve",
    13: "Thirteen",
    14: "Fourteen",
    15: "Fifteen",
    16: "Sixteen",
    17: "Seventeen",
    18: "Eighteen",
    19: "Nineteen",
    20: "Twenty",
    30: "Thirty",
    40: "Forty",
    50: "Fifty",
    60: "Sixty",
    70: "Seventy",
    80: "Eighty",
    90: "Ninety",
    0: "Zero",
    100: "Hundred",
    200: "Two Hundred",
    300: "Three Hundred",
    400: "Four Hundred",
    500: "Five Hundred",
    600: "Six Hundred",
    700: "Seven Hundred",
    800: "Eight Hundred",
    900: "Nine Hundred",
    1000: "Thousand",
    100000: "Hundred Thousand",
}


def num2words(num):
    try:
        result = __numbers_dict[num]
    except KeyError:
        try:
            # Find the decimal(tens) using numb - modulus of number and add the remainder to it
            result = __numbers_dict[num - num % 10] + __numbers_dict[num % 10].lower()
        except KeyError:
            print("Number out of range")
    return result


def digit2words(num):
    try:
        result = __numbers_dict[num]
    except KeyError:
        num_length = len(str(num))
        if num_length == 2:
            # Find the decimal(tens) using numb - modulus of number and add the remainder to it
            result = __numbers_dict[num - num % 10] + __numbers_dict[num % 10].lower()
        elif num_length == 3:
            result = __numbers_dict[num - num % 100] + __numbers_dict[num % 10].lower()
        elif num_length == 4:
            result = __numbers_dict[num - num % 1000] + __numbers_dict[num % 10].lower()
        elif num_length == 5:
            result = (
                __numbers_dict[num - num % 10000] + __numbers_dict[num % 10].lower()
            )
        elif num_length == 6:
            result = (
                __numbers_dict[num - num % 100000] + __numbers_dict[num % 10].lower()
            )
        elif num_length == 7:
            result = (
                __numbers_dict[num - num % 1000000] + __numbers_dict[num % 10].lower()
            )
        elif num_length == 8:
            result = (
                __numbers_dict[num - num % 10000000] + __numbers_dict[num % 10].lower()
            )
        elif num_length == 9:
            result = (
                __numbers_dict[num - num % 100000000]
                + __numbers_dict[num % 100000000].lower()
            )
        else:
            print("Number out of range")

    return result


# def summarize(raw_docx):
# 	""" usage: summarize(yourtext) """

# 	raw_text = raw_docx
# 	docx = TextFrame(raw_text)
# 	stopwords = list(STOPWORDS)
# 	# Build Word Frequency # word.text is tokenization in spacy
# 	word_frequencies = {}
# 	for word in docx.text.split():
# 		if word not in stopwords:
# 			if word not in word_frequencies.keys():
# 				word_frequencies[word] = 1
# 			else:
# 				word_frequencies[word] += 1

# 	print(word_frequencies)
# 	maximum_frequncy = max(word_frequencies.values())

# 	for word in word_frequencies.keys():
# 		word_frequencies[word] = (word_frequencies[word]/maximum_frequncy)
# 	# Sentence Tokens
# 	print(word_frequencies)
# 	sentence_list = [ sentence for sentence in docx.text.split() ]
# 	print(sentence_list)
# 	#Calculate Sentence Scores
# 	sentence_scores = {}
# 	for sent in sentence_list:
# 		for word in sent:
# 			if word.lower() in word_frequencies.keys():
# 				if len(sent.split(' ')) < 5:
# 					if sent not in sentence_scores.keys():
# 						sentence_scores[sent] = word_frequencies[word.lower()]
# 					else:
# 						sentence_scores[sent] += word_frequencies[word.lower()]
# 	print(sentence_scores)
# 	# Find N Largest and Join Sentences
# 	summarized_sentences = nlargest(10, sentence_scores, key=sentence_scores.get)
# 	final_sentences = [ w for w in summarized_sentences ]
# 	print(final_sentences)
# 	print(summarized_sentences)
# 	summary = ' '.join(final_sentences)
# 	return summary


# Source https://github.com/adashofdata/nlp-in-python-tutorial/blob/master/5-Text-Generation.ipynb
def markov_chain(text):
    """Returns a dictionary with each word as
    a key and each value as the list of words that come after the key in the text."""

    # Tokenize the text by word, though including punctuation
    words = text.split(" ")

    # Initialize a default dictionary to hold all of the words and next words
    m_dict = defaultdict(list)

    # Create a zipped list of all of the word pairs and put them in word: list of next words format
    for current_word, next_word in zip(words[0:-1], words[1:]):
        m_dict[current_word].append(next_word)

    # Convert the default dict back into a dictionary
    m_dict = dict(m_dict)
    return m_dict


def __generate_text(chain, count=15):
    """Input a dictionary in the format of key = current word, value = list of next words
    along with the number of words you would like to see in your generated sentence."""

    # Capitalize the first word
    word1 = random.choice(list(chain.keys()))
    sentence = word1.capitalize()

    # Generate the second word from the value list. Set the new word as the first word. Repeat.
    for i in range(count - 1):
        word2 = random.choice(chain[word1])
        word1 = word2
        sentence += " " + word2

    # End it with a period
    sentence += "."
    # print(sentence)
    return sentence


def generate_sentence(text, num_of_words=15):
    """Returns a new sentence/text From a given text using Markov Chains

    Parameters
    ----------
    text : Main Text
    num_of_words : number of words to generate

    Returns
    ----------
    Returns a new sentence of the number of words specified

    Usage
    ------
    >>> from neattext.functions import generate_sentence
    >>> t1 = "your text...here"
    >>> generate_sentence(t1)
    >>> generate_sentence(t1,20)

    #Alternatively generate 4 sentences
    >>> for i in range(4):
            print(generate_sentence(t1))


    """
    result_dict = markov_chain(text)
    final_result_sentence = __generate_text(result_dict, num_of_words)
    return final_result_sentence


def normalize(text, level="shallow"):
    """Normalize Text by converting to lowercase,removing punctuations and square brackets

    Parameters
    ----------
    text : Main Text
    level : level of normalization (shallow/deep)
            shallow:lowercase,remove text in brackets and digits
            deep: shallow + removing puncts,emojis,bad commas,etc
    Returns
    ----------
    Returns a new clean and normalized sentence

    Usage
    ------
    >>> from neattext.functions import normalize
    >>> t1 = "your text...here"
    >>> normalize(t1)

    #Alternatively
    >>> normalize(t1,level='deep')
    """

    if level == "shallow":
        # Lowercase
        text = text.lower()
        # Remove Txt in Square Bracket
        text = re.sub("\\[.*?\\]", "", text)
        text = re.sub("\\w*\\d\\w*", "", text)

    elif level == "deep":
        # Lowercase
        text = text.lower()
        # Remove Txt in Square Bracket
        text = re.sub("\\[.*?\\]", "", text)
        # Remove Digit containing words
        text = re.sub("\\w*\\d\\w*", "", text)
        # Remove Punct
        text = re.sub("[%s]" % re.escape(string.punctuation), "", text)
        # Remove Bad Quotations
        text = re.sub("[‘’“”…]", "", text)
        # Remove emojis
        text = re.sub(EMOJI_REGEX, "", text)

    return text


def fix_contractions(text):
    """Fix contractions in a text"""
    text = text.lower()
    text = re.sub(r"i'm", "i am", text)
    text = re.sub(r"aren't", "are not", text)
    text = re.sub(r"can't", "cannot", text)
    text = re.sub(r"can't've", "cannot have", text)
    text = re.sub(r"'cause", "because", text)
    text = re.sub(r"could've", "could have", text)
    text = re.sub(r"couldn't", "could not", text)
    text = re.sub(r"couldn't've", "could not have", text)
    text = re.sub(r"didn't", "did not", text)
    text = re.sub(r"doesn't", "does not", text)
    text = re.sub(r"don't", "do not", text)
    text = re.sub(r"hadn't", "had not", text)
    text = re.sub(r"hadn't've", "had not have", text)
    text = re.sub(r"hasn't", "has not", text)
    text = re.sub(r"haven't", "have not", text)
    text = re.sub(r"he'd", "he would", text)
    text = re.sub(r"he'd've", "he would have", text)
    text = re.sub(r"he'll", "he will", text)
    text = re.sub(r"he'll've", "he will have", text)
    text = re.sub(r"he's", "he is", text)
    text = re.sub(r"how'd", "how did", text)
    text = re.sub(r"how'd'y", "how do you", text)
    text = re.sub(r"how'll", "how will", text)
    text = re.sub(r"how's", "how is", text)
    text = re.sub(r"I'd", "I would", text)
    text = re.sub(r"I'd've", "I would have", text)
    text = re.sub(r"I'll", "I will", text)
    text = re.sub(r"I'll've", "I will have", text)
    text = re.sub(r"I'm", "I am", text)
    text = re.sub(r"I've", "I have", text)
    text = re.sub(r"isn't", "is not", text)
    text = re.sub(r"it'd", "it had", text)
    text = re.sub(r"it'd've", "it would have", text)
    text = re.sub(r"it'll", "it will", text)
    text = re.sub(r"it'll've", "it will have", text)
    text = re.sub(r"it's", "it is", text)
    text = re.sub(r"let's", "let us", text)
    text = re.sub(r"ma'am", "madam", text)
    text = re.sub(r"mayn't", "may not", text)
    text = re.sub(r"might've", "might have", text)
    text = re.sub(r"mightn't", "might not", text)
    text = re.sub(r"mightn't've", "might not have", text)
    text = re.sub(r"must've", "must have", text)
    text = re.sub(r"mustn't", "must not", text)
    text = re.sub(r"mustn't've", "must not have", text)
    text = re.sub(r"needn't", "need not", text)
    text = re.sub(r"needn't've", "need not have", text)
    text = re.sub(r"o'clock", "of the clock", text)
    text = re.sub(r"oughtn't", "ought not", text)
    text = re.sub(r"oughtn't've", "ought not have", text)
    text = re.sub(r"shan't", "shall not", text)
    text = re.sub(r"sha'n't", "shall not", text)
    text = re.sub(r"shan't've", "shall not have", text)
    text = re.sub(r"she'd", "she would", text)
    text = re.sub(r"she'd've", "she would have", text)
    text = re.sub(r"she'll", "she will", text)
    text = re.sub(r"she'll've", "she will have", text)
    text = re.sub(r"she's", "she is", text)
    text = re.sub(r"should've", "should have", text)
    text = re.sub(r"shouldn't", "should not", text)
    text = re.sub(r"shouldn't've", "should not have", text)
    text = re.sub(r"so've", "so have", text)
    text = re.sub(r"so's", "so is", text)
    text = re.sub(r"that'd", "that would", text)
    text = re.sub(r"that'd've", "that would have", text)
    text = re.sub(r"that's", "that is", text)
    text = re.sub(r"there'd", "there had", text)
    text = re.sub(r"there'd've", "there would have", text)
    text = re.sub(r"there's", "there is", text)
    text = re.sub(r"they'd", "they would", text)
    text = re.sub(r"they'd've", "they would have", text)
    text = re.sub(r"they'll", "they will", text)
    text = re.sub(r"they'll've", "they will have", text)
    text = re.sub(r"they're", "they are", text)
    text = re.sub(r"they've", "they have", text)
    text = re.sub(r"to've", "to have", text)
    text = re.sub(r"wasn't", "was not", text)
    text = re.sub(r"we'd", "we had", text)
    text = re.sub(r"we'd've", "we would have", text)
    text = re.sub(r"we'll", "we will", text)
    text = re.sub(r"we'll've", "we will have", text)
    text = re.sub(r"we're", "we are", text)
    text = re.sub(r"we've", "we have", text)
    text = re.sub(r"weren't", "were not", text)
    text = re.sub(r"what'll", "what will", text)
    text = re.sub(r"what'll've", "what will have", text)
    text = re.sub(r"what're", "what are", text)
    text = re.sub(r"what's", "what is", text)
    text = re.sub(r"what've", "what have", text)
    text = re.sub(r"when's", "when is", text)
    text = re.sub(r"when've", "when have", text)
    text = re.sub(r"where'd", "where did", text)
    text = re.sub(r"where's", "where is", text)
    text = re.sub(r"where've", "where have", text)
    text = re.sub(r"who'll", "who will", text)
    text = re.sub(r"who'll've", "who will have", text)
    text = re.sub(r"who's", "who is", text)
    text = re.sub(r"who've", "who have", text)
    text = re.sub(r"why's", "why is", text)
    text = re.sub(r"why've", "why have", text)
    text = re.sub(r"will've", "will have", text)
    text = re.sub(r"won't", "will not", text)
    text = re.sub(r"won't've", "will not have", text)
    text = re.sub(r"would've", "would have", text)
    text = re.sub(r"wouldn't", "would not", text)
    text = re.sub(r"wouldn't've", "would not have", text)
    text = re.sub(r"y'all", "you all", text)
    text = re.sub(r"y'alls", "you alls", text)
    text = re.sub(r"y'all'd", "you all would", text)
    text = re.sub(r"y'all'd've", "you all would have", text)
    text = re.sub(r"y'all're", "you all are", text)
    text = re.sub(r"y'all've", "you all have", text)
    text = re.sub(r"you'd", "you had", text)
    text = re.sub(r"you'd've", "you would have", text)
    text = re.sub(r"you'll", "you you will", text)
    text = re.sub(r"you'll've", "you you will have", text)
    text = re.sub(r"you're", "you are", text)
    text = re.sub(r"you've", "you have", text)
    text = re.sub(r"he's", "he is", text)
    text = re.sub(r"she's", "she is", text)
    text = re.sub(r"that's", "that is", text)
    text = re.sub(r"what's", "what is", text)
    text = re.sub(r"where's", "where is", text)
    text = re.sub(r"\'ll", " will", text)
    text = re.sub(r"\'ve", " have", text)
    text = re.sub(r"\'re", " are", text)
    text = re.sub(r"\'d", " would", text)
    text = re.sub(r"\'ve", " have", text)
    text = re.sub(r"won't", "will not", text)
    text = re.sub(r"don't", "do not", text)
    text = re.sub(r"did't", "did not", text)
    text = re.sub(r"can't", "can not", text)
    text = re.sub(r"it's", "it is", text)
    text = re.sub(r"couldn't", "could not", text)
    text = re.sub(r"have't", "have not", text)
    # text = re.sub(r"i'm", "i am", text)
    # text = re.sub(r"he's", "he is", text)
    # text = re.sub(r"she's", "she is", text)
    # text = re.sub(r"that's", "that is", text)
    # text = re.sub(r"what's", "what is", text)
    # text = re.sub(r"where's", "where is", text)
    # text = re.sub(r"\'ll", " will", text)
    # text = re.sub(r"\'ve", " have", text)
    # text = re.sub(r"\'re", " are", text)
    # text = re.sub(r"\'d", " would", text)
    # text = re.sub(r"\'ve", " have", text)
    # text = re.sub(r"won't", "will not", text)
    # text = re.sub(r"don't", "do not", text)
    # text = re.sub(r"did't", "did not", text)
    # text = re.sub(r"can't", "can not", text)
    # text = re.sub(r"it's", "it is", text)
    # text = re.sub(r"couldn't", "could not", text)
    # text = re.sub(r"have't", "have not", text)

    return text


def word_freq(text):
    """Returns a term/word frequency"""
    stopwords = list(STOPWORDS)
    # Build Word Frequency(bag) # word.text is tokenization in spacy
    word_frequencies = {}
    for word in text.split():
        if word not in stopwords:
            if word not in word_frequencies.keys():
                word_frequencies[word] = 1
            else:
                word_frequencies[word] += 1

    # print(word_frequencies)
    maximum_frequency = max(word_frequencies.values())

    for word in word_frequencies.keys():
        word_frequencies[word] = word_frequencies[word] / maximum_frequency

    return word_frequencies


def term_freq(text):
    """Returns the Term Frequency of Words in a Sentence

    Definition
    ---------
    The number of times a word appears in a document divided by the total number of words in the document.

    Formular
    ---------
    TF = (Frequency of the word in the sentence) / (Total number of words in the sentence)

    """
    stopwords = list(STOPWORDS)
    # Build Bag of Words or Word Frequency(bag) #
    term_frequencies = {}
    for word in text.split():
        if word not in stopwords:
            if word not in term_frequencies.keys():
                term_frequencies[word] = 1
            else:
                term_frequencies[word] += 1

    maximum_frequency = max(term_frequencies.values())

    for word in term_frequencies.keys():
        term_frequencies[word] = term_frequencies[word] / maximum_frequency

    return term_frequencies


def inverse_df(text):
    """IDF: log((Total number of sentences (documents))/(Number of sentences (documents) containing the word))

    It is important to mention that to mitigate the effect of very rare and very common words on the corpus, the log of the IDF value can be calculated before multiplying it with the TF-IDF value.

    """
    word_idf_values = {}
    total_num_sent = len(text)
    tokenize_word_list = term_freq(text).keys()
    doc_containing_word = 0
    for token in tokenize_word_list:
        for sentence in text:
            if token in sentence.split(" "):
                doc_containing_word += 1
        word_idf_values[token] = math.log((total_num_sent) / (1 + doc_containing_word))

    return word_idf_values


def _lex_richness_herdan(text):
    # Tokenize Word using Words
    tokenized_words = re.split(r"\W+", str(text).lower())
    num_of_word_types = len(set(tokenized_words))
    num_of_word_tokens = len(tokenized_words)
    if num_of_word_types == num_of_word_tokens:
        result_httr = 0
    else:
        # Herdan TTR  log(typ)/log(word_tokens)
        result_httr = math.log(num_of_word_types) / math.log(
            math.sqrt(num_of_word_tokens)
        )

    return result_httr


def _lex_richness_maas_ttr(text):
    # Tokenize Word using Words
    tokenized_words = re.split(r"\W+", str(text).lower())
    num_of_word_types = len(set(tokenized_words))
    num_of_word_tokens = len(tokenized_words)
    if num_of_word_types == num_of_word_tokens:
        result_maas_ttr = 0
    else:
        # Maas TTR (log(w) - log(typ)) / (log(word_tokens) * log(word_tokens)),
        result_maas_ttr = (
            math.log(num_of_word_tokens) - math.log(num_of_word_types)
        ) / (math.log(num_of_word_tokens) * math.log(num_of_word_tokens))

    return result_maas_ttr


def lexical_richness(text):
    """Returns A Dictionary of the Lexical/Text Richness of the text using Type/Token Ratio

            ------------
    ttr: Type Token Ratio
    rttr: Root Type Token Ratio
    cttr: Corrected Type Token Ratio
    httr: Herdan Type Token Ratio
    mttr: Maas Type Token Ratio

    Usage:
    >>> lexical_richness(mytext)

    """

    # Tokenize Word using Words
    tokenized_words = re.split(r"\W+", str(text).lower())
    num_of_word_types = len(set(tokenized_words))
    num_of_word_tokens = len(tokenized_words)

    # TTR typ/word_tokens
    tt_ratio = num_of_word_types / num_of_word_tokens
    # ROOT TTR typ/sqrt(word_tokens)
    root_tt_ratio = num_of_word_types / math.sqrt(num_of_word_tokens)
    # Corrected TTR typ/sqrt(2 * word_tokens)
    corrected_tt_ratio = num_of_word_types / math.sqrt(2 * num_of_word_tokens)
    # Herdan TTR  log(typ)/log(word_tokens)
    herdan_tt_ratio = _lex_richness_herdan(text)
    # Mass TTR (log(w) - log(typ)) / (log(word_tokens) * log(word_tokens)),
    mass_ttr = _lex_richness_maas_ttr(text)

    result = {
        "ttr": tt_ratio,
        "rttr": root_tt_ratio,
        "cttr": corrected_tt_ratio,
        "httr": herdan_tt_ratio,
        "mttr": mass_ttr,
    }
    return result


def word_length_freq(text):
    """Returns the Count/Freq Distribution of Words by their length.
    Useful for stylometric mendelhall curve

    >>> word_length_freq(mytext)

    """
    all_tokens_length = [len(token) for token in text.split()]
    count_of_n_length_word = Counter(all_tokens_length)
    sorted_count_of_n_length_word = sorted(dict(count_of_n_length_word).items())

    return dict(sorted_count_of_n_length_word)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['neattext',
 'neattext.explainer',
 'neattext.functions',
 'neattext.pattern_data']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'neattext',
    'version': '0.1.1',
    'description': 'Neattext - a simple NLP package for cleaning text',
    'long_description': '# neattext\nNeatText:a simple NLP package for cleaning textual data and text preprocessing\n\n[![Build Status](https://travis-ci.org/Jcharis/neattext.svg?branch=master)](https://travis-ci.org/Jcharis/neattext)\n\n[![GitHub license](https://img.shields.io/github/license/Jcharis/neattext)](https://github.com/Jcharis/neattext/blob/master/LICENSE)\n\n#### Problem\n+ Cleaning of unstructured text data\n+ Reduce noise [special characters,stopwords]\n+ Reducing repetition of using the same code for text preprocessing\n\n#### Solution\n+ convert the already known solution for cleaning text into a reuseable package\n\n#### Docs\n+ Check out the full docs [here](https://jcharis.github.io/neattext/)\n\n#### Installation\n```bash\npip install neattext\n```\n\n### Usage\n+ The OOP Way(Object Oriented Way)\n+ NeatText offers 4 main classes for working with text data\n\t- TextFrame : a frame-like object for cleaning text\n\t- TextCleaner: remove or replace specifics\n\t- TextExtractor: extract unwanted text data\n\t- TextMetrics: word stats and metrics\n\n### Overall Components of NeatText\n![](images/neattext_features_jcharistech.png)\n\n### Using TextFrame\n+ Keeps the text as `TextFrame` object. This allows us to do more with our text. \n+ It inherits the benefits of the TextCleaner and the TextMetrics out of the box with some additional features for handling text data.\n+ This is the simplest way for text preprocessing with this library alternatively you can utilize the other classes too.\n\n\n```python\n>>> import neattext as nt \n>> mytext = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊."\n>>> docx = nt.TextFrame(text=mytext)\n>>> docx.text \n"This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊."\n>>>\n>>> docx.describe()\nKey      Value          \nLength  : 73             \nvowels  : 21             \nconsonants: 34             \nstopwords: 4              \npunctuations: 8              \nspecial_char: 8              \ntokens(whitespace): 10             \ntokens(words): 14             \n>>> \n>>> docx.length\n73\n>>> # Scan Percentage of Noise(Unclean data) in text\n>>> d.noise_scan()\n{\'text_noise\': 19.17808219178082, \'text_length\': 73, \'noise_count\': 14}\n>>> \n>>> docs.head(16)\n\'This is the mail\'\n>>> docx.tail()\n>>> docx.count_vowels()\n>>> docx.count_stopwords()\n>>> docx.count_consonants()\n>>> docx.nlongest()\n>>> docx.nshortest()\n>>> docx.readability()\n```\n#### Basic NLP Task (Tokenization,Ngram,Text Generation)\n```python\n>>> docx.word_tokens()\n>>>\n>>> docx.sent_tokens()\n>>>\n>>> docx.term_freq()\n>>>\n>>> docx.bow()\n```\n#### Basic Text Preprocessing\n```python\n>>> docx.normalize()\n\'this is the mail example@gmail.com ,our website is https://example.com 😊.\'\n>>> docx.normalize(level=\'deep\')\n\'this is the mail examplegmailcom our website is httpsexamplecom \'\n\n>>> docx.remove_puncts()\n>>> docx.remove_stopwords()\n>>> docx.remove_html_tags()\n>>> docx.remove_special_characters()\n>>> docx.remove_emojis()\n>>> docx.fix_contractions()\n```\n\n##### Handling Files with NeatText\n+ Read txt file directly into TextFrame\n```python\n>>> import neattext as nt \n>>> docx_df = nt.read_txt(\'file.txt\')\n```\n+ Alternatively you can instantiate a TextFrame and read a text file into it\n```python\n>>> import neattext as nt \n>>> docx_df = nt.TextFrame().read_txt(\'file.txt\')\n```\n\n##### Chaining Methods on TextFrame\n```python\n>>> t1 = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊 and it will cost $100 to subscribe."\n>>> docx = TextFrame(t1)\n>>> result = docx.remove_emails().remove_urls().remove_emojis()\n>>> print(result)\n\'This is the mail  ,our WEBSITE is   and it will cost $100 to subscribe.\'\n```\n\n\n\n#### Clean Text\n+ Clean text by removing emails,numbers,stopwords,emojis,etc\n+ A simplified method for cleaning text by specifying as True/False what to clean from a text\n```python\n>>> from neattext.functions import clean_text\n>>> \n>>> mytext = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊."\n>>> \n>>> clean_text(mytext)\n\'mail example@gmail.com ,our website https://example.com .\'\n```\n+ You can remove punctuations,stopwords,urls,emojis,multiple_whitespaces,etc by setting them to True.\n\n+ You can choose to remove or not remove punctuations by setting to True/False respectively\n\n```python\n>>> clean_text(mytext,puncts=True)\n\'mail example@gmailcom website https://examplecom \'\n>>> \n>>> clean_text(mytext,puncts=False)\n\'mail example@gmail.com ,our website https://example.com .\'\n>>> \n>>> clean_text(mytext,puncts=False,stopwords=False)\n\'this is the mail example@gmail.com ,our website is https://example.com .\'\n>>> \n```\n+ You can also remove the other non-needed items accordingly\n```python\n>>> clean_text(mytext,stopwords=False)\n\'this is the mail example@gmail.com ,our website is https://example.com .\'\n>>>\n>>> clean_text(mytext,urls=False)\n\'mail example@gmail.com ,our website https://example.com .\'\n>>> \n>>> clean_text(mytext,urls=True)\n\'mail example@gmail.com ,our website .\'\n>>> \n\n```\n\n#### Removing Punctuations [A Very Common Text Preprocessing Step]\n+ You remove the most common punctuations such as fullstop,comma,exclamation marks and question marks by setting most_common=True which is the default\n+ Alternatively you can also remove all known punctuations from a text.\n```python\n>>> import neattext as nt \n>>> mytext = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊. Please don\'t forget the email when you enter !!!!!"\n>>> docx = nt.TextFrame(mytext)\n>>> docx.remove_puncts()\nTextFrame(text="This is the mail example@gmailcom our WEBSITE is https://examplecom 😊 Please dont forget the email when you enter ")\n\n>>> docx.remove_puncts(most_common=False)\nTextFrame(text="This is the mail examplegmailcom our WEBSITE is httpsexamplecom 😊 Please dont forget the email when you enter ")\n```\n\n#### Removing Stopwords [A Very Common Text Preprocessing Step]\n+ You can remove stopwords from a text by specifying the language. The default language is English\n+ Supported Languages include English(en),Spanish(es),French(fr)|Russian(ru)|Yoruba(yo)|German(de)\n\n```python\n>>> import neattext as nt \n>>> mytext = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊. Please don\'t forget the email when you enter !!!!!"\n>>> docx = nt.TextFrame(mytext)\n>>> docx.remove_stopwords(lang=\'en\')\nTextFrame(text="mail example@gmail.com ,our WEBSITE https://example.com 😊. forget email enter !!!!!")\n```\n\n\n#### Remove Emails,Numbers,Phone Numbers,Dates,Btc Address,VisaCard Address,etc \n```python\n>>> print(docx.remove_emails())\n>>> \'This is the mail  ,our WEBSITE is https://example.com 😊.\'\n>>>\n>>> print(docx.remove_stopwords())\n>>> \'This mail example@gmail.com ,our WEBSITE https://example.com 😊.\'\n>>>\n>>> print(docx.remove_numbers())\n>>> docx.remove_phone_numbers()\n>>> docx.remove_btc_address()\n```\n\n\n#### Remove Special Characters\n```python\n>>> docx.remove_special_characters()\n```\n\n#### Remove Emojis\n```python\n>>> print(docx.remove_emojis())\n>>> \'This is the mail example@gmail.com ,our WEBSITE is https://example.com .\'\n```\n\n\n#### Remove Custom Pattern\n+ You can also specify your own custom pattern, incase you cannot find what you need in the functions using the `remove_custom_pattern()` function\n```python\n>>> import neattext.functions as nfx \n>>> ex = "Last !RT tweeter multiple &#7777"\n>>> \n>>> nfx.remove_custom_pattern(e,r\'&#\\d+\')\n\'Last !RT tweeter multiple  \'\n\n\n\n```\n\n#### Replace Emails,Numbers,Phone Numbers\n```python\n>>> docx.replace_emails()\n>>> docx.replace_numbers()\n>>> docx.replace_phone_numbers()\n```\n\n#### Chain Multiple Methods\n```python\n>>> t1 = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊 and it will cost $100 to subscribe."\n>>> docx = TextCleaner(t1)\n>>> result = docx.remove_emails().remove_urls().remove_emojis()\n>>> print(result)\n\'This is the mail  ,our WEBSITE is   and it will cost $100 to subscribe.\'\n\n```\n\n### Using TextExtractor\n+ To Extract emails,phone numbers,numbers,urls,emojis from text\n```python\n>>> from neattext import TextExtractor\n>>> docx = TextExtractor()\n>>> docx.text = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊."\n>>> docx.extract_emails()\n>>> [\'example@gmail.com\']\n>>>\n>>> docx.extract_emojis()\n>>> [\'😊\']\n```\n\n\n### Using TextMetrics\n+ To Find the Words Stats such as counts of vowels,consonants,stopwords,word-stats\n```python\n>>> from neattext import TextMetrics\n>>> docx = TextMetrics()\n>>> docx.text = "This is the mail example@gmail.com ,our WEBSITE is https://example.com 😊."\n>>> docx.count_vowels()\n>>> docx.count_consonants()\n>>> docx.count_stopwords()\n>>> docx.word_stats()\n```\n\n### Usage \n+ The MOP(method/function oriented way) Way\n\n```python\n>>> from neattext.functions import clean_text,extract_emails\n>>> t1 = "This is the mail example@gmail.com ,our WEBSITE is https://example.com ."\n>>> clean_text(t1,puncts=True,stopwords=True)\n>>>\'this mail examplegmailcom website httpsexamplecom\'\n>>> extract_emails(t1)\n>>> [\'example@gmail.com\']\n```\n\n+ Alternatively you can also use this approach\n```python\n>>> import neattext.functions as nfx \n>>> t1 = "This is the mail example@gmail.com ,our WEBSITE is https://example.com ."\n>>> nfx.clean_text(t1,puncts=True,stopwords=True)\n>>>\'this mail examplegmailcom website httpsexamplecom\'\n>>> nfx.extract_emails(t1)\n>>> [\'example@gmail.com\']\n```\n\n### Explainer\n+ Explain an emoji or unicode for emoji \n\t- emoji_explainer()\n\t- emojify()\n\t- unicode_2_emoji()\n\n\n```python\n>>> from neattext.explainer import emojify\n>>> emojify(\'Smiley\')\n>>> \'😃\'\n```\n\n```python\n>>> from neattext.explainer import emoji_explainer\n>>> emoji_explainer(\'😃\')\n>>> \'SMILING FACE WITH OPEN MOUTH\'\n```\n\n```python\n>>> from neattext.explainer import unicode_2_emoji\n>>> unicode_2_emoji(\'0x1f49b\')\n\t\'FLUSHED FACE\'\n```\n\n\n\n### Documentation\nPlease read the [documentation](https://github.com/Jcharis/neattext/wiki) for more information on what neattext does and how to use is for your needs.You can also check \nout our readthedocs page [here](https://jcharis.github.io/neattext/)\n\n\n### More Features To Add\n+ basic nlp task\n+ currency normalizer\n\n#### Acknowledgements\n+ Inspired by packages like `clean-text` from Johannes Fillter and `textify` by JCharisTech\n\n\n#### NB\n+ Contributions Are Welcomed\n+ Notice a bug, please let us know.\n+ Thanks A lot\n\n\n#### By\n+ Jesse E.Agbe(JCharis)\n+ Jesus Saves @JCharisTech\n\n\n\n',
    'author': 'Jesse E.Agbe(JCharis)',
    'author_email': 'jcharistech@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Jcharis/neattext',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.3,<4.0',
}


setup(**setup_kwargs)
